"""Init file for the device types."""

from .ZendureABPack import ZendureABPack
from .ZendureSolarflowHub import ZendureSolarflowHub
from .ZendureAio import ZendureAio
from .ZendureHyper import ZendureHyper
from .ZendureAce import ZendureAce

__all__ = (
    "ZendureABPack",
    "ZendureSolarflowHub",
    "ZendureAio",
    "ZendureHyper",
    "ZendureAce",
)



import json
import logging
import sys
import requests
from requests.adapters import HTTPAdapter

FORMAT = "%(asctime)s:%(levelname)s: %(message)s"
logging.basicConfig(stream=sys.stdout, level="INFO", format=FORMAT)
log = logging.getLogger("")


class ZendureHttpAPI:
    """An API class to handle communication with the Zendure API"""

    def __init__(self, parameter=None):
        self.parameter = parameter
        self.session = None

    def __enter__(self):
        self.session = requests.Session()
        adapter = HTTPAdapter(max_retries=3)
        self.session.verify = True
        self.session.mount("http://", adapter)
        self.session.mount("https://", adapter)
        self.session.headers = {
            "Content-Type": "application/json",
            "Accept-Language": "de-DE",
            "appVersion": "4.3.1",
            "User-Agent": "Zendure/4.3.1 (iPhone; iOS 14.4.2; Scale/3.00)",
            "Accept": "*/*",
            "Authorization": "Basic Q29uc3VtZXJBcHA6NX4qUmRuTnJATWg0WjEyMw==",
            "Blade-Auth": "bearer (null)",
        }
        return self

    def __exit__(self, type, value, traceback):
        self.session.close()
        self.session = None

    def authenticate(self, username, password):
        authBody = {
            "password": password,
            "account": username,
            "appId": "121c83f761305d6cf7e",
            "appType": "iOS",
            "grantType": "password",
            "tenantId": "",
        }

        try:
            url = f'{self.parameter["solarFlowTokenUrl"]}'
            log.info("Authenticating with Zendure ...")
            response = self.session.post(url=url, json=authBody)
            if response.ok:
                respJson = response.json()
                token = respJson["data"]["accessToken"]
                self.session.headers["Blade-Auth"] = f"bearer {token}"
                return token
            else:
                log.error("Authentication failed!")
                log.error(response.text)
        except Exception as e:
            log.error(e)

    def get_device_list(self):
        try:
            url = f'{self.parameter["solarFlowQueryDeviceListUrl"]}'
            log.info("Getting device list ...")
            response = self.session.post(url=url)
            if response.ok:
                respJson = response.json()
                return respJson["data"]
            else:
                log.error("Fetching device list failed!")
                log.error(response.text)
        except Exception as e:
            log.error(e)



"""The base device class."""

# -*- coding: utf-8 -*-

from __future__ import print_function

import logging
import sys
from paho.mqtt import client as mqtt_client

FORMAT = "%(asctime)s:%(levelname)s: %(message)s"
logging.basicConfig(stream=sys.stdout, level="INFO", format=FORMAT)
log = logging.getLogger("")


class SolarflowDeviceBase:
    """The Solarflow Device class."""

    _mqttClient: mqtt_client.Client = None

    id = None
    deviceKey: str = None
    snNumber: str = None
    namev = None
    productId = None
    productKey: str = None
    onlineFlag = None
    productName: str = None
    wifiStatus = None
    blueState = None
    fourGStatus = None
    isShareFlag = None
    input = None
    output = None
    soc = None  # electricity / electricLevel
    restState = None
    upsMode = None
    upgradeStatusDes = None
    productType = None
    upgradeStatus = None
    bindId = None
    bindStatus = None
    batteryCode = None
    packList = None
    inputPower = None
    outputPower = None
    slowChargePower = None
    temperature = None
    temperatureUnit = None
    remainOutTime = None
    bindType = None
    seriesMode = None
    parallelMode = None
    networkType = None
    standard = None
    isSwitch = None
    url = None
    remainOil = None
    genMode = None
    shortCode = None
    phaseCheck = None
    passMode = None  # pass
    clusterList = None
    clusterNotice = None
    nodeStatus = None
    clusterId = None
    phase = None
    reverseState = None

    def __init__(self, data, mqttclient):
        """Create an entity base object."""
        if data is not None:
            self.name = data["name"]
            self.productName = data["productName"]
            self.productKey = data["productKey"]
            self.deviceKey = data["deviceKey"]
            self.soc = data["electricity"]
            self.id = data["id"]
            self.snNumber = data["snNumber"]
            self.productId = data["productId"]
            self.onlineFlag = data["onlineFlag"]
            self.wifiStatus = data["wifiStatus"]
            self.blueState = data["blueState"]
            self.fourGStatus = data["fourGStatus"]
            self.isShareFlag = data["isShareFlag"]
            self.input = data["input"]
            self.output = data["output"]
            self.electricity = data["electricity"]
            self.restState = data["restState"]
            self.upsMode = data["upsMode"]
            self.upgradeStatusDes = data["upgradeStatusDes"]
            self.productType = data["productType"]
            self.upgradeStatus = data["upgradeStatus"]
            self.bindId = data["bindId"]
            self.bindStatus = data["bindStatus"]
            self.batteryCode = data["batteryCode"]
            self.packList = data["packList"]
            self.inputPower = data["inputPower"]
            self.outputPower = data["outputPower"]
            self.slowChargePower = data["slowChargePower"]
            self.temperature = data["temperature"]
            self.temperatureUnit = data["temperatureUnit"]
            self.remainOutTime = data["remainOutTime"]
            self.bindType = data["bindType"]
            self.seriesMode = data["seriesMode"]
            self.parallelMode = data["parallelMode"]
            self.networkType = data["networkType"]
            self.standard = data["standard"]
            self.isSwitch = data["isSwitch"]
            self.url = data["url"]
            self.remainOil = data["remainOil"]
            self.genMode = data["genMode"]
            self.shortCode = data["shortCode"]
            self.phaseCheck = data["phaseCheck"]
            self.clusterList = data["clusterList"]
            self.clusterNotice = data["clusterNotice"]
            self.nodeStatus = data["nodeStatus"]
            self.clusterId = data["clusterId"]
            self.phase = data["phase"]
            self.reverseState = data["reverseState"]
            self.passMode = data["pass"]
        else:
            log.error("Error creating device, data is empty!")

    def __repr__(self):
        """Return a string."""
        return "{productName}: {name}".format(
            productName=self.productName,
            name=self.name,
        )

    def _update(self, data):
        log.debug("Update base device" + self.__repr__())
        if "name" in data:
            self.name = data["name"]

        if "electricLevel" in data:
            self.soc = data["electricLevel"]

        if "wifiState" in data:
            self.wifiStatus = data["wifiState"]

    def _requestUpdate(self):
        if self.productKey and self.deviceKey:
            self._mqttClient.publish(
                f"iot/{self.productKey}/{self.deviceKey}/properties/read",
                '{"properties": ["getAll"]}',
            )



"""The Solarflow Hub Device class."""

# -*- coding: utf-8 -*-

import json
import logging
import sys
from typing import Dict

from .ZendureABPack import ZendureABPack
from .ZendureDeviceBase import SolarflowDeviceBase

FORMAT = "%(asctime)s:%(levelname)s: %(message)s"
logging.basicConfig(stream=sys.stdout, level="INFO", format=FORMAT)
log = logging.getLogger("")


class ZendureHyper(SolarflowDeviceBase):
    """The Solarflow Hub Device class."""

    outputPackPower: int = 0
    outputHomePower: int = 0
    packInputPower: int = 0
    solarPower1: int = 0
    solarPower2: int = 0
    solarInputPower: int = 0
    packNum: int = 0
    outputLimit: int = 0
    inputLimit: int = 0
    inverseMaxPower: int = 0
    buzzerSwitch: int = 0
    minSoc: int = 0
    socSet: int = 0
    remainOutTime: int = 59940
    remainInputTime: int = 59940
    inputMode: int = 0
    passMode: int = 1
    autoRecover: int = 0
    gridPower: int = 0
    acSwitch: int = 0
    acMode: int = 0

    """ masterSwitch = 0
    wifiState = 0
    socSet = 0
    solarInputPower = 0
    packInputPower = 0
    outputPackPower = 74
    outputHomePower = 46
    outputLimit = 800    
    packState = 1
    hubState = 1
    masterSoftVersion = 8232
    masterhaerVersion = 0
    inputMode = 0
    blueOta = 1
    pvBrand = 1
    minSoc = 50    
    autoModel = 0
    gridPower = 0
    smartMode = 0
    smartPower = 0
    passMode = 1
    autoRecover = 0
    masterSwitch = 1
    buzzerSwitch = 0
    socSet = 1000
    solarInputPower = 115
    packInputPower = 0
    outputPackPower = 67
    outputHomePower = 44
    outputLimit = 800
    inputLimit = 0
    remainOutTime = 604
    remainInputTime = 59940
    packState = 1
    hubState = 1
    masterSoftVersion = 8232
    masterhaerVersion = 0
    inputMode = 0
    blueOta = 1
    pvBrand = 1
    autoModel = 0
    gridPower = 0
    smartMode = 0
    smartPower = 0 """

    packData: Dict[str, ZendureABPack] = {}

    def _update(self, data):
        super()._update(data)
        log.debug("Update Zendure Solarflow Hub " + self.__repr__())

        if "outputPackPower" in data:
            self.outputPackPower = data["outputPackPower"]

            if self.outputPackPower > 0:
                self.packInputPower = 0

        if "outputHomePower" in data:
            self.outputHomePower = data["outputHomePower"]

        if "solarPower1" in data:
            self.solarPower1 = data["solarPower1"]

        if "solarPower2" in data:
            self.solarPower2 = data["solarPower2"]

        if "solarInputPower" in data:
            self.solarInputPower = data["solarInputPower"]

        if "packInputPower" in data:
            self.packInputPower = data["packInputPower"]

            if self.packInputPower > 0:
                self.outputPackPower = 0

        if "packNum" in data:
            self.packNum = data["packNum"]

        if "outputLimit" in data:
            self.outputLimit = data["outputLimit"]

        if "inputLimit" in data:
            self.inputLimit = data["inputLimit"]

        if "inverseMaxPower" in data:
            self.inverseMaxPower = data["inverseMaxPower"]

        if "buzzerSwitch" in data:
            self.buzzerSwitch = data["buzzerSwitch"]

        if "minSoc" in data:
            self.minSoc = data["minSoc"] / 10

        if "socSet" in data:
            self.socSet = data["socSet"] / 10

        if "remainOutTime" in data:
            self.remainOutTime = data["remainOutTime"]

        if "remainInputTime" in data:
            self.remainInputTime = data["remainInputTime"]

        if "inputMode" in data:
            self.inputMode = data["inputMode"]

        if "passMode" in data:
            self.passMode = data["passMode"]

        if "autoRecover" in data:
            self.autoRecover = data["autoRecover"]

        if "gridPower" in data:
            self.gridPower = data["gridPower"]

        if "acMode" in data:
            self.acMode = data["acMode"]

    def _updatePackData(self, packdata):
        if packdata is None:
            return

        log.debug("Update Pack for Zendure Solarflow Hub " + self.__repr__())

        for _pack in packdata:
            if _pack["sn"] in self.packData.keys():
                # Pack is already in list
                pack = self.packData[_pack["sn"]]
                pack._update(pack)
            else:
                # Pack is not in list
                pack = ZendureABPack(_pack)
                self.packData[_pack["sn"]] = pack

    def setAcmode(self, acMode: int):
        if (
            self.productKey
            and self.deviceKey
            and acMode
            and acMode >= 0
            and acMode <= 2
        ):
            outputlimitdata = {"properties": {"acMode": acMode}}
            self._mqttClient.publish(
                f"iot/{self.productKey}/{self.deviceKey}/properties/write",
                json.dumps(outputlimitdata),
            )

    def setAcSwitch(self, value: bool):
        acSwitch = {"properties": {"acSwitch": 0 if not value else 1}}
        self._mqttClient.publish(
            f"iot/{self.productKey}/{self.deviceKey}/properties/write",
            json.dumps(acSwitch),
        )

    def setOutputlimit(self, outputlimit: int):
        if self.productKey and self.deviceKey and outputlimit:
            outputlimitdata = {"properties": {"outputLimit": outputlimit}}
            self._mqttClient.publish(
                f"iot/{self.productKey}/{self.deviceKey}/properties/write",
                json.dumps(outputlimitdata),
            )

    def setInputlimit(self, inputlimit: int):
        if self.productKey and self.deviceKey and inputlimit:
            inputlimitdata = {"properties": {"outputLimit": inputlimit}}
            self._mqttClient.publish(
                f"iot/{self.productKey}/{self.deviceKey}/properties/write",
                json.dumps(inputlimitdata),
            )

    def setBuzzerSwitch(self, value: bool):
        buzzerSwitch = {"properties": {"buzzerSwitch": 0 if not value else 1}}
        self._mqttClient.publish(
            f"iot/{self.productKey}/{self.deviceKey}/properties/write",
            json.dumps(buzzerSwitch),
        )

    def setDischargeLimit(self, dischargelimit: int):
        dischargelimitdata = {"properties": {"minSoc": dischargelimit * 10}}
        self._mqttClient.publish(
            f"iot/{self.productKey}/{self.deviceKey}/properties/write",
            json.dumps(dischargelimitdata),
        )

    def setChargeLimit(self, chargelimit: int):
        chargelimitdata = {"properties": {"socSet": chargelimit * 10}}
        self._mqttClient.publish(
            f"iot/{self.productKey}/{self.deviceKey}/properties/write",
            json.dumps(chargelimitdata),
        )



import logging
import sys
import json
import base64
from typing import Dict
from paho.mqtt import client as mqtt_client

from PyZendureSolarflow.ZendureHttpApi import ZendureHttpAPI
from PyZendureSolarflow.devicetypes import (
    ZendureAce,
    ZendureAio,
    ZendureHyper,
    ZendureSolarflowHub,
)

FORMAT = "%(asctime)s:%(levelname)s: %(message)s"
logging.basicConfig(stream=sys.stdout, level="INFO", format=FORMAT)
log = logging.getLogger("")

"""Connection Parameter"""

hostname = "app.zendure.tech"
versionGlobal = "v2"
versionEu = "eu"
solarFlowDevRegisterPath = "developer/api/apply"
solarFlowTokenPath = "auth/app/token"
solarFlowDeviceListPath = "productModule/device/queryDeviceListByConsumerId"

parameterEu = {
    "solarFlowDevRegisterUrl": f"https://{hostname}/{versionEu}/{solarFlowDevRegisterPath}",
    "solarFlowTokenUrl": f"https://{hostname}/{versionEu}/{solarFlowTokenPath}",
    "solarFlowQueryDeviceListUrl": f"https://{hostname}/{versionEu}/{solarFlowDeviceListPath}",
    "mqttUrl": "mqtteu.zen-iot.com",
    "mqttPort": 1883,
    "mqttUser": "zenApp",
    "mqttPassword": "SDZzJGo5Q3ROYTBO",
}

parameterGlobal = {
    "solarFlowDevRegisterUrl": f"https://{hostname}/{versionGlobal}/{solarFlowDevRegisterPath}",
    "solarFlowTokenUrl": f"https://{hostname}/{versionGlobal}/{solarFlowTokenPath}",
    "solarFlowQueryDeviceListUrl": f"https://{hostname}/{versionGlobal}/{solarFlowDeviceListPath}",
    "mqttUrl": "mq.zen-iot.com",
    "mqttPort": 1883,
    "mqttUser": "zenApp",
    "mqttPassword": "b0sjUENneTZPWnhk",
}


class PyZendureSolarflow:
    _token = None
    _location_: str = None
    _username: str = None
    _password: str = None
    _client: mqtt_client.Client = None
    _devices: Dict[str, any] = None
    _templates: Dict[str, any] = None

    def __init__(self, username, password, location="global", debug=False):
        """Create a PyZendureSolarflow object."""
        self._location = location
        self._username = username
        self._password = password

        if debug == True:
            log.setLevel(level="DEBUG")

    def login(self):
        if not self._location or not self._username or not self._password:
            log.error("Missing parameters!")
            return

        if self._location == "eu":
            self._parameter = parameterEu
        else:
            self._parameter = parameterGlobal

        with ZendureHttpAPI(parameter=self._parameter) as zendureHttpApi:
            self._token = zendureHttpApi.authenticate(self._username, self._password)

            if self._devices is None:
                self._devices = {}

            if self._token:
                self._client = self.connect_zendure_mqtt(
                    token=self._token,
                    mqttUrl=self._parameter["mqttUrl"],
                    mqttPort=self._parameter["mqttPort"],
                    mqttUser=self._parameter["mqttUser"],
                    mqttPassword=base64.b64decode(
                        self._parameter["mqttPassword"]
                    ).decode("utf-8"),
                )

                devicelist = zendureHttpApi.get_device_list()
                for _device in devicelist:
                    device = None
                    if _device["productKey"] == "8bM93H":
                        # ACE 1500
                        device = ZendureAce(_device, self._client)
                        self._devices[device.deviceKey] = device
                    elif _device["productKey"] == "73bkTV":
                        # HUB 1200
                        device = ZendureSolarflowHub(_device, self._client)
                        self._devices[device.deviceKey] = device
                    elif _device["productKey"] == "A8yh63":
                        # HUB 2000
                        device = ZendureSolarflowHub(_device, self._client)
                        self._devices[device.deviceKey] = device
                    elif _device["productKey"] == "yWF7hV":
                        # AIO
                        device = ZendureAio(_device, self._client)
                        self._devices[device.deviceKey] = device
                    elif _device["productKey"] == "s3Xk4x":
                        # Smart Plug EU
                        device = None
                    elif _device["productKey"] == "gDa3tb":
                        # Hyper
                        device = ZendureHyper(_device, self._client)
                        self._devices[device.deviceKey] = device

                    if self._client and device:
                        self.subscribe(device.productKey, device.deviceKey)

                self._client.loop_start()
            else:
                log.error("Missing token!")

    def connect_zendure_mqtt(
        self, token, mqttUrl, mqttPort, mqttUser, mqttPassword
    ) -> mqtt_client:
        _mqtt_client = mqtt_client.Client(
            client_id=token, userdata="Zendure Production MQTT"
        )
        _mqtt_client.username_pw_set(username=mqttUser, password=mqttPassword)
        _mqtt_client.reconnect_delay_set(min_delay=1, max_delay=120)
        _mqtt_client.on_connect = self.on_connect
        _mqtt_client.on_disconnect = self.on_disconnect
        _mqtt_client.connect(mqttUrl, mqttPort)
        return _mqtt_client

    def subscribe(self, productKey: str, deviceKey: str):
        report_topic = f"/{productKey}/{deviceKey}/#"
        iot_topic = f"iot/{productKey}/{deviceKey}/#"
        self._client.subscribe(report_topic)
        self._client.subscribe(iot_topic)
        self._client.on_message = self.on_message

    def on_connect(self, client: mqtt_client.Client, userdata, flags, rc):
        if rc == 0:
            log.info(f"Connected to MQTT Broker: {userdata}")
        else:
            log.error("Failed to connect, return code %d\n", rc)

    def on_disconnect(self, client: mqtt_client.Client, userdata, rc):
        if rc != 0:
            log.warning("Disconnected from Zendure MQTT: " + str(rc))
            client.loop_stop()
            client.disconnect()

    def on_message(self, client: mqtt_client.Client, userdata, msg):
        payload = json.loads(msg.payload.decode())

        topicSplitted = msg.topic.split("/")
        deviceKey = topicSplitted[2]

        device = self._devices[deviceKey]

        if device:
            if "properties/report" in msg.topic and "properties" in payload:
                device._update(payload["properties"])

            if "packData" in payload:
                packdata = payload["packData"]
                if len(packdata) > 0:
                    device._updatePackData(packdata)



"""The Solarflow Battery class."""

# -*- coding: utf-8 -*-

from __future__ import print_function

import logging
import sys

FORMAT = "%(asctime)s:%(levelname)s: %(message)s"
logging.basicConfig(stream=sys.stdout, level="INFO", format=FORMAT)
log = logging.getLogger("")


class ZendureABPack:
    """The Solarflow Battery class."""

    sn: str = None
    type: str = None
    socLevel: int = 0
    power: int = 0
    maxTemp: int = 0
    minVol: float = 0.0
    maxVol: float = 0.0
    totalVol: float = 0.0

    def __init__(self, data):
        """Create an entity base object."""
        if data is None:
            return

        self._update(data)

    def __repr__(self):
        """Return a string."""
        return "{type}: {sn}".format(
            type=self.type,
            sn=self.sn,
        )

    def _update(self, data):
        if "sn" in data.keys():
            self.sn = data["sn"]
            if self.sn[0] == "C":
                # It's a AB2000
                self.type = "AB2000"

            elif self.sn[0] == "A":
                # It's a AB1000
                self.type = "AB1000"

        if "socLevel" in data:
            self.socLevel = data["socLevel"]

        if "power" in data:
            self.power = data["power"]

        if "maxTemp" in data:
            self.maxTemp = data["maxTemp"]

        if "minVol" in data:
            self.minVol = data["minVol"]

        if "maxVol" in data:
            self.maxVol = data["maxVol"]

        if "totalVol" in data:
            self.totalVol = data["totalVol"]

        log.debug("Updated battery " + self.__repr__())



#!/usr/bin/env python
"""A simple CLI tool."""
# -*- coding: utf-8 -*-

from __future__ import print_function
import configparser
import logging
import argparse
import sys

from PyZendureSolarflow.PyZendureSolarflow import PyZendureSolarflow

FORMAT = "%(asctime)s:%(levelname)s: %(message)s"
logging.basicConfig(stream=sys.stdout, level="INFO", format=FORMAT)
log = logging.getLogger("")
config: configparser.ConfigParser


def main(args=None):
    """Enter the main function of the CLI tool."""
    parser = argparse.ArgumentParser(description="Zendure Solarflow CLI tool.")
    parser.add_argument(
        "-v", action="store_true", dest="verbose", help="be more verbose"
    )
    parser.add_argument(
        "-l",
        "--location",
        type=str,
        dest="location",
        help="Zendure Account Location",
        default="global",
    )
    parser.add_argument(
        "-u", "--username", type=str, dest="username", help="Zendure Account Username"
    )
    parser.add_argument(
        "-p", "--password", type=str, dest="password", help="Zendure Account Password"
    )

    args = parser.parse_args(args)
    config = load_config()

    username = config.get("global", "username", fallback=None) or args.username

    password = config.get("global", "password", fallback=None) or args.password

    location = config.get("global", "location", fallback=None) or args.location

    logging.basicConfig()
    if args.verbose:
        logging.getLogger("PyZendureSolarflow").setLevel(logging.DEBUG)

    zendureapi = None
    try:
        zendureapi = PyZendureSolarflow(
            username=username, password=password, location=location
        )
        zendureapi.login()
    finally:
        log.error("PyZendureSolarflow stopped!")


def load_config():
    config = configparser.ConfigParser()
    try:
        with open("config.ini", "r") as cf:
            config.read_file(cf)
    except:
        log.info("No configuration file (config.ini) found! Using input args")
    return config


if __name__ == "__main__":
    main()
